/******************************************************************************
 *
 *          AtomSoft Clock Using PIC18F248, DS1306, 74LS164, 74LS165
 *
 ******************************************************************************
 * FileName:        main.c
 * Author:          Jason Lopez
 * Processor:       PIC18F248(0)
 * Compiler:        C18
 * Company:         AtomSoftTech
 * Version:         1.0.0
 ******************************************************************************/
#include <p18F248.h>
#include <stdio.h>
#include <delays.h>
#include <spi.h>

#pragma config WDT = OFF, LVP = OFF, OSC = HS
/************************************
Prototypes
*************************************/
void main(void);
void high_vector (void);
void initSPI(void);
void e_togg(void);
void lcd_line(char line);
void lcd_cmd(unsigned char letter);
void lcd_char(unsigned char letter);
void lcd_string(char *senpoint);
void lcd_init(void);
void delay_ms(int mS);
void delay_us(int uS);
void delay_s(int S);
void lcdByte(unsigned char dNyb,unsigned char rs);
char RTCRegRead(char adx);
void RTCRegWrite(char adx, char data);
void showMenu(void);
void setDate(void);
void setTime(char type);
static unsigned char uint2bcd(unsigned char ival);
char readBtn(void);
unsigned char bcd2dec(unsigned char aBCD, char type);
char dec2lcd(char type, char dec);
void putDate(char type, char data);
void alarm (void);

/************************************
Definitions
************************************/
#define CS LATCbits.LATC2
#define CS_T  TRISCbits.TRISC2
#define SCL_T TRISCbits.TRISC3
#define SDA_T TRISCbits.TRISC4
#define SDO_T TRISCbits.TRISC5

#define LCD_DAT_T TRISCbits.TRISC0
#define LCD_CLK_T TRISCbits.TRISC1

#define LCD_DAT LATCbits.LATC0
#define LCD_CLK LATCbits.LATC1

#define LCD_E_T TRISCbits.TRISC6
#define LCD_E   LATCbits.LATC6

#define PL  LATBbits.LATB0
#define CP2 LATBbits.LATB1
#define SDL  PORTBbits.RB2

#define PL_T   TRISBbits.TRISB0
#define CP2_T  TRISBbits.TRISB1
#define SDL_T  TRISBbits.TRISB2

#define setupBtn  0x08
#define downBtn   0x04
#define upBtn     0x01
#define selectBtn 0x10

#define alarmOut  LATBbits.LATB3
#define alarmOutT TRISBbits.TRISB3
#define alarmIn   TRISBbits.TRISB4
/************************************
Variables
************************************/
char string[] =  "                ";
char string2[] = "                ";

unsigned char time[16];
unsigned char lDate[16];
unsigned char theDay;
unsigned char theDate[2];
unsigned char theMonth[2];
unsigned char theYear[2];

unsigned char theHour[2];
unsigned char theMin[2];
unsigned char theSec[2];

unsigned char decHour;
unsigned char decMin;
unsigned char decSec;
unsigned char decAMPM;

unsigned char decDay;
unsigned char decDate;
unsigned char decMonth;
unsigned char decYear;
unsigned char myTmp;
/************************************
Main
************************************/
void main(void){
    unsigned char tmp,tmp2;
    unsigned char buff[40];
    char i;

    unsigned char view = 0;

    time[16] = 0;
    ADCON1 = 0x0E;

    initSPI();
	lcd_init();

    PL_T = 0;
    CP2_T = 0;
    SDL_T = 1;

/*  Setup Alarm Bits  */
    alarmIn = 1;
    alarmOutT = 0;
    alarmOut = 0;

/*  Setup Interrupts  */

    RCONbits.IPEN = 1;
    PIR1 = PIR2 = 0;
    INTCON2 = 0x81;
    INTCON = 0xC8;


	lcd_line(1);
	sprintf(string,"  AtomSoftTech",0);
	lcd_string(string);

	lcd_line(2);
	sprintf(string," 2-Wire 74LS164",0);
	lcd_string(string);

    delay_s(1);

	lcd_line(2);
	sprintf(string,"  DS1306 TEST.  ",0);
    lcd_string(string);

    RTCRegWrite(0x8F,0x01);
    delay_s(1);

    while(1){
 
    tmp = RTCRegRead(0x02);
    time[15] = (tmp >> 4) & 0x02;

    if(time[15] == 0) 
        time[15] = 'A' ;
    else 
        time[15] = 'P';

    time[14] = 0x20;

    tmp = RTCRegRead(0x00);
    decSec = tmp;
    theSec[1] = time[13] = (tmp & 0x0F) + 0x30;
    theSec[0] = time[12] = ((tmp >> 4) & 0x0F ) + 0x30;
    time[11] = ':';

    tmp = RTCRegRead(0x01);
    decMin = tmp;
    theMin[1] = time[10] = (tmp & 0x0F) + 0x30;
    theMin[0] = time[9] = ((tmp >> 4) & 0x0F ) + 0x30;
    time[8] = ':';

    tmp = RTCRegRead(0x02);
    decHour = tmp;
    theHour[1] = time[7] = (tmp & 0x0F) + 0x30;
    theHour[0] = time[6] = ((tmp >> 4) & 0x01 ) + 0x30;

    time[5] = 0x20;
    time[4] = ':';
    time[3] = 'e';
    time[2] = 'm';
    time[1] = 'i';
    time[0] = 'T';

	lcd_line(2);
	lcd_string(time);
/////////////////////////////
    tmp = RTCRegRead(0x03);
    putDate('d', tmp);

    tmp = RTCRegRead(0x04);
    lDate[10] = (tmp & 0x0F) + 0x30;
    lDate[9] = ((tmp >> 4) & 0x03 ) + 0x30;

    tmp = RTCRegRead(0x05);
    putDate('m', bcd2dec(tmp,3));

    tmp = RTCRegRead(0x06);
    lDate[12] = '2';
    lDate[13] = '0';
    lDate[15] = (tmp & 0x0F) + 0x30;;
    lDate[14] = (tmp >> 4) + 0x30;
/////////////////////////////

	lcd_line(1);
    if(view == 0){
    	sprintf(string,"  AtomSoftTech  ",0);
	    lcd_string(string);
    } else {
	    lcd_string(&lDate);
    }

    decDay   = RTCRegRead(0x03);
    decDate  = RTCRegRead(0x04);
    decMonth = RTCRegRead(0x05);
    decYear  = RTCRegRead(0x06);

    tmp = readBtn();

    switch(tmp){
        case setupBtn:
            showMenu();
            break;
        case upBtn:
            view=1;
            break;
        case downBtn:
            view=0;
            break;
        case selectBtn:
            alarmOut = 0;
            break;
    }

    delay_ms(10);

    }
}
/***********************************
RTC Tools
************************************
Convert Char(byte) to BCD
************************************/
unsigned char uint2bcd(unsigned char ival)
{
	return ((ival / 10) << 4) | (ival % 10);
}
/************************************
Convert BCD to Char(byte)
************************************/
unsigned char bcd2dec(unsigned char aBCD, char type)
{
    char lowNyb = aBCD & 0x0F;
    char highNyb = aBCD >> 4;
    char MyDec = lowNyb;
    char x;

    switch(type){
        case 0:
            highNyb &= 0x0F;
            break;
        case 1:
            highNyb &= 0x01;
            break;
        case 2:
            highNyb &= 0x03;
            break;
        case 3:
            highNyb &= 0x07;
            break;
        case 4:
            highNyb &= 0x00;
            break;
    }

    for(x=0;x<highNyb;x++){
        MyDec += 10;    
    }

    return MyDec;
}
/************************************
Read a Register off the DS1306
************************************/
char RTCRegRead(char adx){
    char tmp;
    CS = 1;
        WriteSPI(adx); //sec
        tmp = ReadSPI();
    CS = 0;

    return tmp;
}
/************************************
Write to a Register on the DS1306
************************************/
void RTCRegWrite(char adx, char data){
    adx |= 0x80;

    CS = 1;
        WriteSPI(adx);
        WriteSPI(data);
    CS = 0;
}
/************************************
Initialize SPI
************************************/
void initSPI(void){
    CS_T = 0;       //CS is output
    SCL_T = 0;      //SCL is output
    SDA_T = 1;      //SDA is input
    SDO_T = 0;      //SDO is output

    OpenSPI(SPI_FOSC_4,MODE_10,SMPEND);         

    CS = 1;
        WriteSPI(0x8F);
        WriteSPI(0x00);
    CS = 0;

}
/***********************************
LCD Functions
************************************
Send String to LCD
************************************/
void lcd_string(char *senpoint){
    delay_ms(1);
	while(*senpoint != '\0'){
		lcd_char(*senpoint);
		senpoint++;
	}
}
/************************************
Send Data(Nybble) to LCD
************************************/
void lcdByte(unsigned char dNyb,unsigned char rs){
	int i;

	LCD_DAT=0;							//Clear 74LS164 set initial bit to 0
	for(i=0;i<8;i++){				    //repeat for 8 bits
		LCD_CLK=1;LCD_CLK=0;			//write 0's to the 164
	}

	for(i=0;i<4;i++){				    //output the nybble
		if((dNyb & 0x08) != 0)
			LCD_DAT=1;
		else
			LCD_DAT=0;

		LCD_CLK=1;LCD_CLK=0;
		dNyb=dNyb<<1;
	}

	LCD_DAT = rs;						    //output the RS bit value
	LCD_CLK=1;LCD_CLK=0;

	LCD_DAT = 0;
	LCD_CLK=1;LCD_CLK=0;
	LCD_CLK=1;LCD_CLK=0;
	LCD_CLK=1;LCD_CLK=0;

	e_togg();
}
/************************************
Toggle E Line
************************************/
void e_togg(void){
	LCD_E=1;LCD_E=0;
}
/************************************
Set LCD Line
************************************/
void lcd_line(char line){
    if(line == 0x01)
        lcd_cmd(0x80);
    else
	    lcd_cmd(0xc0);
}
/************************************
LCD Send Command
************************************/
void lcd_cmd(unsigned char letter){
	unsigned char temp;
	temp=letter;
	temp=temp>>4;
	lcdByte(temp,0);
	temp=letter;
	temp=temp&0x0f;
	lcdByte(temp,0);
}
/************************************
LCD Send Character
************************************/
void lcd_char(unsigned char letter){
	unsigned char temp;
	temp=letter;
	temp=temp>>4;
	lcdByte(temp,1);
	temp=letter;
	temp=temp&0x0f;
	lcdByte(temp,1);
}
/************************************
LCD Initialization
************************************/
void lcd_init(void){

    LCD_E_T = 0;
    LCD_CLK_T = 0;
    LCD_DAT_T = 0;

	lcdByte(0x03,0);
	delay_ms(5);
	e_togg();
	delay_us(160);
	e_togg();
	delay_us(160);
	lcdByte(0x02,0);
	delay_us(160);
	lcd_cmd(0x28);					//set 4-bit mode and 2 lines
	delay_us(160);
	lcd_cmd(0x10);					//cursor move & shift left
	delay_us(160);
	lcd_cmd(0x06);					//entry mode = increment
	delay_us(160);
	lcd_cmd(0x0d);					//display on - cursor blink on
	delay_us(160);
	lcd_cmd(0x01);					//clear display
	delay_ms(500);
}
/************************************
Delays (S,mS,uS) for 20Mhz
************************************/
/*********************
    Delay Second(s)
**********************/
void delay_s(int S){
    int y;
    char x;

    for(y=0;y<S;y++)
        for(x=0;x<4;x++)
            delay_ms(250);

}
/************************
    Delay MilliSecond(s)
*************************/
void delay_ms(int mS){
    int y;

    for(y=0;y<mS;y++)
        Delay1KTCYx(5);
}
/************************
    Delay MicroSeconds
*************************
Lowercase 'u' is common
symbol for Micro
*************************/
void delay_us(int uS){
    int y;
    char x;

    for(y=0;y<uS;y++)
        for(x=0;x<5;x++)
            Nop();
}
/***********************************
SN74LS165 Functions
************************************
Toggle Clock Pin
************************************/
void clk_togg(void){
	LCD_CLK=1;LCD_CLK=0;
}
/************************************
readBtn
************************************/
char readBtn(void){
    char x;
    char buff;
    delay_ms(100);
    PL=0;  PL=1;

    for(x=0;x<8;x++){
        buff <<= 1;             // shift composed byte by 1
        buff &= 0xFE;           // clear bit 0

        if (SDL)                 // is data line high
            buff |= 0x01;       // set bit 0 to logic 1

        CP2 = 0; CP2 = 1;
    }
    delay_ms(100);
    return buff & 0x1F;
}
/***********************************
LCD / RTC stuff
************************************
Show Menu
************************************/
void showMenu(void){
    char item, isEnter;
    isEnter = 0;
    item = 1;   //Default to first menu item

while(1){
/******************************
    Which item are we viewing
*******************************/
    switch(item){
        case 0:
            item=3; //user went do down too far start from 3
            break;
        case 1:
            sprintf(string2,"1.Set Time      ",0);
            break;
        case 2:
            sprintf(string2,"2.Set Date      ",0);
            break;
        case 3:
            sprintf(string2,"3.Set Alarm     ",0);
            break;
        case 4:
            item=1; //user went too up far start them from 1
            break;
    }
        
/*****************************
    Determine Button Pressed
******************************/
    switch(readBtn()){
        case upBtn:
            item++;
            break;
        case downBtn:
            item--;
            break;
        case selectBtn:
            isEnter = 1;
            break;
        case setupBtn:
            return;
            break;
    }
/*****************************
    Enter is pressed
******************************/
    if(isEnter == 1){
        if(item==1)
            setTime('t');

        if(item==2){
            setDate();
        }
        if(item==3){
            setTime('a');
        }

        item=9; //a overkill number incase i want to add more functions.
    }
/*************************
    Show Menu to the user
**************************/
    lcd_line(1);
    sprintf(string,"    Settings    ",0);
    lcd_string(&string);

    lcd_line(2);
    lcd_string(&string2);
/**************************
    Check if user is done.
***************************/
    if(item==9)
        break; //leave while(1) loop

}//end of while loop
}
/****************
    Set the Time
****************/
void setTime(char type){
    char bcdMin,bcdHour;
    char x = 0;

    decMin = bcd2dec(decMin,3);
    decHour = bcd2dec(decHour,1);
    decAMPM = (RTCRegRead(0x01) >> 5) & 0x01 ;
/********************
    Main while loop
*********************/    
while(1){
/**********************************
    Determine which title to show
***********************************/    
switch(type){
    case 'a':
        switch(x){
            case 0:
                sprintf(string,"Set Alarm: Hour ",0);
                break;
            case 1:
                sprintf(string,"Set Alarm: Min  ",0);
                break;
            case 2:
                sprintf(string,"Set Alarm: AM/PM",0);
                break;
        }
        break;
    case 't':
        switch(x){
            case 0:
                sprintf(string,"Set Time: Hour  ",0);
                break;
            case 1:
                sprintf(string,"Set Time: Min   ",0);
                break;
            case 2:
                sprintf(string,"Set Time: AM/PM ",0);
                break;
        }
        break;
}
/*****************************************
   Button pressed functions
******************************************/    
    switch(readBtn()){
        case upBtn:
            switch(x){
                case 0:
                    decHour++;
                    break;
                case 1:
                    decMin++;
                    break;
                case 2:
                    decAMPM++;
                    break;
            }
            break;
        case downBtn:
            switch(x){
                case 0:
                    decHour--;
                    break;
                case 1:
                    if(decMin == 0)
                        decMin = 59;
                    else
                        decMin--;
                    break;
                case 2:
                    decAMPM--;
                    break;
            }
            break;
        case selectBtn:
            x++;
            break;
        case setupBtn:
            return;         //User Pressed Setup aka Cancel

    }
/**********************************************
    Error Fix's:
***********************************************
ex: If hour is greater than 12 then set to 1
if its a 0 set it to 12 since this is 12 hr
there is no 00:00 hour only 1-12 AM/PM
***********************************************/    
    if(decHour >= 13)
        decHour = 1;

    if(decHour == 0)
        decHour = 12;

    if(decMin >= 60)
        decMin = 0;

    if(decAMPM >= 2)
        decAMPM = 0;

    if(decAMPM < 0)
        decAMPM = 0;

    if(x >= 3)
        break;

/*****************************************
    Convert to BCD for the RTC(DS1306)
******************************************/   

    bcdHour = uint2bcd(decHour) | 0x40;     //0x40 = 12 Hour Mode
    if(decAMPM >= 1)
        bcdHour = bcdHour | 0x20; //0b00100000

    bcdMin = uint2bcd(decMin);
/*********************************************
    Convert to seperate numbers or split BCD
    and add 0x30 so it can be displayed on
    LCD.
**********************************************/   

    time[6] = ((bcdHour >> 4) & 0x01) + 0x30;
    time[7] = (bcdHour & 0x0F) + 0x30;

    time[9] = dec2lcd('h',decMin);
    time[10] = dec2lcd('l',decMin);

    time[9] = (bcdMin >> 4) + 0x30;
    time[10] = (bcdMin & 0x0F) + 0x30;

    time[12] = time[13] = 0x30;

    if(decAMPM == 1)
        time[15] = 'P';
    else
        time[15] = 'A';
/*****************************************
    Update the LCD with new info.
******************************************/   
	lcd_line(1);
	lcd_string(string);
	
    lcd_line(2);
	lcd_string(&time);
}
/*****************************************
    Done! Now write to RTC the new Data
    Determine if Time or Alarm
******************************************/   
    switch(type){
        case 't':
            RTCRegWrite(0x80,0x00);     //Seconds = 00
            RTCRegWrite(0x81,bcdMin);   //Minutes
            RTCRegWrite(0x82,bcdHour);  //Hours
            break;
        case 'a':
            RTCRegWrite(0x87,0x00);     //Seconds = 00
            RTCRegWrite(0x88,bcdMin);   //Minutes
            RTCRegWrite(0x89,bcdHour);  //Hours
            RTCRegWrite(0x8A,0x80);     //Alarm goes on when hours, minutes, and seconds match
            break;
    }
}
/**********************************
            Set the Date
***********************************/
void setDate(void){
    char bcdDay,bcdDate,bcdMonth,bcdYear;
    char x = 0;

    decDay = bcd2dec(decDay,1);
    decDate = bcd2dec(decDate,1);
    decMonth = bcd2dec(decMonth,1);
    decYear = bcd2dec(decYear,0);
/********************
    Main while loop
*********************/    
while(1){
/**********************************
    Determine which title to show
***********************************/    
        switch(x){
            case 0:
                sprintf(string,"Set Date: Day   ",0);
/*****************************************
    Put the Day in LCD Format
******************************************/    
                putDate( 'd', decDay );
                break;
            case 1:
                sprintf(string,"Set Date: Date  ",0);
/*****************************************
    Put the Date in LCD Format
******************************************/
                lDate[9] = dec2lcd('u',decDate);
                lDate[10] = dec2lcd('l',decDate);
                break;
            case 2:
                sprintf(string,"Set Date: Month ",0);
/*****************************************
    Put the Month in LCD Format
******************************************/    
                putDate( 'm', decMonth );
                break;
            case 3:
                sprintf(string,"Set Date: Year  ",0);
/*****************************************
    Put the Year in LCD Format
******************************************/    
                lDate[12] = '2';
                lDate[13] = '0';
                lDate[14] = dec2lcd('u',decYear);
                lDate[15] = dec2lcd('l',decYear);
                break;
        }

/*****************************************
   Button pressed functions
******************************************/    
    switch(readBtn()){
        case upBtn:
            switch(x){
                case 0:
                    decDay++;
                    break;
                case 1:
                    decDate++;
                    break;
                case 2:
                    decMonth++;
                    break;
                case 3:
                    decYear++;
                    break;
            }
            break;
        case downBtn:
            switch(x){
                case 0:
                    decDay--;
                    break;
                case 1:
                    decDate--;
                    break;
                case 2:
                    decMonth--;
                    break;
                case 3:
                    decYear--;
                    break;
            }
            break;
        case selectBtn:
            x++;
            break;
        case setupBtn:
            return;         //User Pressed Setup aka Cancel

    }
/**********************************************
    Error Fix's:
***********************************************
ex: If hour is greater than 12 then set to 1
if its a 0 set it to 12 since this is 12 hr
there is no 00:00 hour only 1-12 AM/PM
***********************************************/    
    if(decDay >= 8)
        decDay = 1;

    if(decDay == 0)
        decDay = 7;
//////////////////////////
    if(decDate >= 32)
        decDate = 1;

    if(decDate == 0)
        decDate = 31;
//////////////////////////
    if(decMonth >= 13)
        decMonth = 1;

    if(decMonth == 0)
        decMonth = 12;
//////////////////////////
    if(x >= 4)
        break;

/*****************************************
    Convert to BCD for the RTC(DS1306)
******************************************/ 
    bcdDay = uint2bcd(decDay);
    bcdDate = uint2bcd(decDate);
    bcdMonth = uint2bcd(decMonth);
    bcdYear = uint2bcd(decYear);

/*****************************************
    Update the LCD with new info.
******************************************/   
	lcd_line(1);
	lcd_string(string);
	
    lcd_line(2);
	lcd_string(&lDate);

}
/*****************************************
    Done! Now write to RTC the new Data
    Determine if Time or Alarm
******************************************/   
    RTCRegWrite(0x83,(bcdDay & 0x07));       //The Day Sunday(01) to Saturday(07)
    RTCRegWrite(0x84,(bcdDate & 0x3F));     //Date from 01-31
    RTCRegWrite(0x85,(bcdMonth & 0x3F));    //Month From Jan(01) to Dec(12)
    RTCRegWrite(0x86, bcdYear);             //The Year from 2000(01) to 2099(99) 2100(00). 

}
/*****************************************
    Load Date or Month in Variable
    for LCD Usage
******************************************/   
void putDate(char type, char data){
    switch(type){
        case 'd':
            switch(data){
                case 1:
                    sprintf(lDate,"Sun.            ",0);
                    break;
                case 2:
                    sprintf(lDate,"Mon.            ",0);
                    break;
                case 3:
                    sprintf(lDate,"Tue.            ",0);
                    break;
                case 4:
                    sprintf(lDate,"Wed.            ",0);
                    break;
                case 5:
                    sprintf(lDate,"Thu.            ",0);
                    break;
                case 6:
                    sprintf(lDate,"Fri.            ",0);
                    break;
                case 7:
                    sprintf(lDate,"Sat.            ",0);
                    break;
            }
            break;
        case 'm':
            switch(data){
                case 1:
                    lDate[5] = 'J';lDate[6] = 'a';lDate[7] = 'n';
                    break;
                case 2:
                    lDate[5] = 'F';lDate[6] = 'e';lDate[7] = 'b';
                    break;
                case 3:
                    lDate[5] = 'M';lDate[6] = 'a';lDate[7] = 'r';
                    break;
                case 4:
                    lDate[5] = 'A';lDate[6] = 'p';lDate[7] = 'r';
                    break;
                case 5:
                    lDate[5] = 'M';lDate[6] = 'a';lDate[7] = 'y';
                    break;
                case 6: 
                    lDate[5] = 'J';lDate[6] = 'u';lDate[7] = 'n';
                    break;
                case 7:
                    lDate[5] = 'J';lDate[6] = 'u';lDate[7] = 'l';
                    break;
                case 8:
                    lDate[5] = 'A';lDate[6] = 'u';lDate[7] = 'g';
                    break;
                case 9:
                    lDate[5] = 'S';lDate[6] = 'e';lDate[7] = 'p';
                    break;
                case 10:
                    lDate[5] = 'O';lDate[6] = 'c';lDate[7] = 't';
                    break;
                case 11:
                    lDate[5] = 'N';lDate[6] = 'o';lDate[7] = 'v';
                    break;
                case 12:
                    lDate[5] = 'D';lDate[6] = 'e';lDate[7] = 'c';
                    break;
            }
            lDate[8] = '.';
            break;
    }
}
/*****************************************
    Convert a Decimal To LCD 
*****************************************
    h = HIGH PART   -    l = LOW PART
*****************************************
    Converts to BCD then Splits BCD 
    returning HIGH/LOW in a char    
******************************************/   
char dec2lcd(char type, char dec){
    char tmp;
    tmp = uint2bcd(dec);

    switch(type){
        case 'u':
            tmp = (tmp >> 4) + 0x30;        //High Nybble
            break;
        case 'l':
            tmp = (tmp & 0x0F) + 0x30;      //Low  Nybble
            break;
    }

    return tmp;     
}
/*****************************************
    Interrupt Handler
*****************************************/
#pragma code high_vector_section=0x8
void high_vector (void)
{
    _asm GOTO alarm _endasm
}
/*****************************************
    Alarm Interrupt code
*****************************************/
#pragma interrupt alarm
void alarm (void){
    alarmOut = 1;
    myTmp = PORTB;
    INTCON = 0xC8;
}